@@ -23,22 +23,24 @@ crew_controller_local <- function(
   tls = crew::crew_tls(),
   tls_enable = NULL,
   tls_config = NULL,
-  seconds_interval = 0.5,
+  seconds_interval = 1,
   seconds_timeout = 60,
   seconds_launch = 30,
   seconds_idle = 300,
   seconds_wall = Inf,
   seconds_exit = NULL,
-  retry_tasks = TRUE,
+  retry_tasks = NULL,
   tasks_max = Inf,
   tasks_timers = 0L,
   reset_globals = TRUE,
   reset_packages = FALSE,
   reset_options = FALSE,
   garbage_collection = FALSE,
-  crashes_error = 5L,
+  crashes_error = NULL,
   launch_max = NULL,
   r_arguments = c("--no-save", "--no-restore"),
+  crashes_max = 5L,
+  backup = NULL,
   options_metrics = crew::crew_options_metrics(),
   options_local = crew::crew_options_local(),
   local_log_directory = NULL,
@@ -72,28 +74,42 @@ crew_controller_local <- function(
     name = "launch_max",
     date = "2024-11-04",
     version = "0.10.1.9000",
-    alternative = "crashes_error",
+    alternative = "none",
     condition = "warning",
     value = launch_max
   )
+  crew_deprecate(
+    name = "crashes_error",
+    date = "2025-01-13",
+    version = "0.10.2.9002",
+    alternative = "none",
+    condition = "message",
+    value = crashes_error
+  )
+  crew_deprecate(
+    name = "retry_tasks",
+    date = "2025-01-24",
+    version = "0.10.2.9005",
+    alternative = "none",
+    condition = "message",
+    value = retry_tasks
+  )
   options_local$log_directory <- local_log_directory %|||%
     options_local$log_directory
   options_local$log_join <- local_log_join %|||%
     options_local$log_join
   client <- crew_client(
-    name = name,
-    workers = workers,
     host = host,
     port = port,
     tls = tls,
     tls_enable = tls_enable,
     tls_config = tls_config,
     seconds_interval = seconds_interval,
-    seconds_timeout = seconds_timeout,
-    retry_tasks = retry_tasks
+    seconds_timeout = seconds_timeout
   )
   launcher <- crew_launcher_local(
     name = name,
+    workers = workers,
     seconds_interval = seconds_interval,
     seconds_timeout = seconds_timeout,
     seconds_launch = seconds_launch,
@@ -105,7 +121,6 @@ crew_controller_local <- function(
     reset_packages = reset_packages,
     reset_options = reset_options,
     garbage_collection = garbage_collection,
-    crashes_error = crashes_error,
     tls = tls,
     r_arguments = r_arguments,
     options_metrics = options_metrics,
@@ -113,7 +128,12 @@ crew_controller_local <- function(
     local_log_directory = local_log_directory,
     local_log_join = local_log_join
   )
-  controller <- crew_controller(client = client, launcher = launcher)
+  controller <- crew_controller(
+    client = client,
+    launcher = launcher,
+    crashes_max = crashes_max,
+    backup = backup
+  )
   controller$validate()
   controller
 }
