@@ -10,7 +10,9 @@ deparse_safe <- function(expr, collapse = "\n", backtick = TRUE) {
   out
 }
 
-deparse_control_custom <- .deparseOpts(c("keepNA", "keepInteger"))
+deparse_control_custom <- .deparseOpts(
+  c("keepNA", "keepInteger", "niceNames")
+)
 
 deparse_direct <- function(...) {
   produce_deparse_direct()(...)
