@@ -30,7 +30,7 @@ if_any <- function(condition, true, false) {
   }
 }
 
-crew_null <- structure(list(emptyenv()), class = "crew_null")
+crew_null <- structure(list("crew_null"), class = "crew_null")
 
 is_crew_null <- function(x) {
   class(x)[1L] == "crew_null" # For performance, we avoid inherits(). # nolint
