@@ -14,12 +14,14 @@ crew_test("crew_client() active bindings", {
   on.exit(client$terminate())
   expect_equal(client$host, "127.0.0.1")
   expect_equal(client$port, 123L)
+  expect_true(inherits(client$tls, "crew_class_tls"))
   expect_equal(client$seconds_interval, 123)
   expect_equal(client$seconds_timeout, 456)
-  expect_false(client$retry_tasks)
-  expect_true(inherits(client$tls, "crew_class_tls"))
+  expect_false(client$started)
+  expect_null(client$url)
+  expect_null(client$profile)
   expect_null(client$client)
-  expect_null(client$throttle)
+  expect_null(client$dispatcher)
   expect_silent(client$validate())
 })
 
@@ -32,24 +34,16 @@ crew_test("crew_client() works", {
     rm(client)
     crew_test_sleep()
   })
-  expect_null(client$started)
+  expect_false(client$started)
   expect_null(client$dispatcher)
+  expect_null(client$url)
+  expect_equal(client$resolved(), 0L)
   expect_silent(client$start())
+  expect_silent(client$validate())
   expect_true(client$started)
-  log <- client$summary()
-  expect_equal(
-    sort(colnames(log)),
-    sort(
-      c(
-        "worker",
-        "online",
-        "instances",
-        "assigned",
-        "complete",
-        "socket"
-      )
-    )
-  )
+  url <- client$url
+  expect_true(is.character(url) && length(url) == 1L)
+  expect_true(nzchar(url) && !anyNA(url))
   expect_s3_class(client$client, "ps_handle")
   expect_s3_class(client$dispatcher, "ps_handle")
   expect_equal(length(client$dispatcher), 1L)
@@ -59,40 +53,30 @@ crew_test("crew_client() works", {
     seconds_interval = 0.01,
     seconds_timeout = 30
   )
-  socket <- log$socket
-  expect_true(is.character(socket) && length(socket) > 0L)
-  expect_true(nzchar(socket) && !anyNA(socket))
-  expect_equal(length(socket), 1L)
-  expect_false(log$online)
-  expect_equal(log$assigned, 0L)
-  expect_equal(log$complete, 0L)
-  expect_equal(log$instances, 0L)
   bin <- if_any(tolower(Sys.info()[["sysname"]]) == "windows", "R.exe", "R")
   path <- file.path(R.home("bin"), bin)
-  call <- sprintf("mirai::daemon('%s')", socket)
+  call <- sprintf("mirai::daemon('%s', dispatcher = TRUE)", url)
   px <- processx::process$new(command = path, args = c("-e", call))
   crew_retry(
     ~{
-      daemons <- mirai::status(.compute = client$name)$daemons
       identical(
-        as.integer(unname(daemons[, "online", drop = TRUE])),
+        as.integer(mirai::status(.compute = client$profile)$connections),
         1L
       )
     },
     seconds_interval = 0.5,
     seconds_timeout = 60
   )
-  m <- mirai::mirai(ps::ps_pid(), .compute = client$name)
+  task <- mirai::mirai(ps::ps_pid(), .compute = client$profile)
   crew_retry(
-    ~!mirai::unresolved(m),
+    ~!nanonext::.unresolved(task),
     seconds_interval = 0.5,
     seconds_timeout = 10
   )
-  expect_false(unresolved(m))
-  expect_true(is.numeric(m$data))
-  expect_true(abs(m$data - ps::ps_pid()) > 0.5)
+  expect_false(nanonext::.unresolved(task))
+  expect_true(is.numeric(task$data))
+  expect_true(abs(task$data - ps::ps_pid()) > 0.5)
   expect_true(client$started)
-  expect_true(client$summary()$online)
   expect_silent(client$start())
   for (index in seq_len(2L)) {
     expect_silent(client$terminate())
@@ -104,13 +88,13 @@ crew_test("crew_client() works", {
     )
   }
   px$signal(signal = crew_terminate_signal())
-  expect_null(client$summary())
 })
 
 crew_test("crew_client() cover a line", {
   client <- crew_client(host = "127.0.0.1")
   private <- crew_private(client)
   private$.started <- TRUE
+  private$.profile <- "abc"
   expect_null(client$terminate())
 })
 
